    /***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.splash;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import javax.inject.Provider;
import java.io.IOException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import it.tidalwave.util.thread.ThreadType;
import it.tidalwave.util.thread.annotation.ThreadConfined;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.mobile.asset.SmartAssetManager;
import it.tidalwave.mobile.asset.SmartAssetManager.ProgressListener;
import it.tidalwave.bluebill.mobile.preferences.TaxonomyPreferences;
import it.tidalwave.bluebill.mobile.news.NewsService;
import it.tidalwave.mobile.ui.QuestionWithFeedback;
import static org.openide.util.NbBundle.*;

/***********************************************************************************************************************
 *
 * @stereotype Controller
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class SplashActivityController
  {
    private static final Logger log = LoggerFactory.getLogger(SplashActivityController.class); 

    private static final Class<SplashActivityController> _ = SplashActivityController.class;
    
    /* package */ static final int PROGRESS_BAR_ADVANCE_DELAY = 200;
    /* package */ static final int PROGRESS_BAR_ADVANCE_STEP = 5;

    @Nonnull
    private final SplashActivityView view;

    private volatile boolean initializationCompleted = false;

    private final Provider<NewsService> newsService = Locator.createProviderFor(NewsService.class);

    private final Provider<TaxonomyPreferences> taxonomyPreferences = Locator.createProviderFor(TaxonomyPreferences.class);
   
    private final Provider<SmartAssetManager> assetManager = Locator.createProviderFor(SmartAssetManager.class);
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public SplashActivityController (final @Nonnull SplashActivityView view)
      {
        this.view = view;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final Runnable progressAnimator = new Runnable()
      {
        private int progress = 0;

        @Override
        public void run()
          {
            while (!initializationCompleted)
              {
                progress = (progress <= 100 - PROGRESS_BAR_ADVANCE_STEP) ? progress + PROGRESS_BAR_ADVANCE_STEP : 0;
                view.setInitializationProgress(progress);

                try
                  {
                    Thread.sleep(PROGRESS_BAR_ADVANCE_DELAY);
                  }
                catch (InterruptedException e)
                  {
                    break;
                  }
              }
          }
      };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final ProgressListener progressListener = new ProgressListener()
      {
        public void notifyProgress (final @Nonnegative int percent, final @Nonnegative long byteCount)
          {
            view.setInitializationProgress(percent);
          }
      };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final Runnable initializer = new Runnable()
      {
        @Override
        public void run()
          {
            try
              {
                startNewsService();
                initializeAssets();
                loadTaxonomy();
                
                log.info("Initialization complete");
                view.renderMainUI();
              }
            catch (OutOfMemoryError e)
              {
                log.error("Fatal error during initialization: {}", e.toString()); // FIXME ii8n
                log.error("", e);
                view.notifyFatalError(new QuestionWithFeedback("Fatal error", "Out of memory")
                  {
                    @Override
                    public void onConfirm()
                      {
                        view.dismiss();
                      }
                  }); 
              }
            catch (Throwable t)
              {
                log.error("Fatal error during initialization: {}", t.toString()); // FIXME ii8n
                log.error("", t);
                view.notifyFatalError(new QuestionWithFeedback("Fatal error", "Internal error")
                  {
                    @Override
                    public void onConfirm()
                      {
                        view.dismiss();
                      }
                  });
              }
          }
      };


    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public void runInitialization()
      {
        new Thread(initializer).start();
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.NOT_UI)
    private void startNewsService()
      {
        log.info("starting news service...");
        newsService.get().start();
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.NOT_UI)
    private void initializeAssets()
      throws IOException
      {
        log.info("initializing assets...");
        view.setInitializationText(getMessage(_, "initAssets"));
        assetManager.get().installAsset("xeno-canto.zip", progressListener);
        assetManager.get().installAsset("factsheets.zip", progressListener);
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.NOT_UI)
    private void loadTaxonomy()
      {
        log.info("loading the taxonomy...");
        view.setInitializationText(getMessage(_, "initChecklist"));
        new Thread(progressAnimator).start();
        taxonomyPreferences.get().getTaxonomy(); // long task
        initializationCompleted = true;
      }
  }
